import { DimensionType } from '@graphpolaris/shared/lib/schema';

const isNumerical = (value: any): boolean => {
  if (typeof value === 'number') return true;
  const parsedNumber = parseFloat(value);
  if (!isNaN(parsedNumber)) return true;
  return false;
};

const isSpatial = (value: any): boolean => {
  if (Array.isArray(value) && value.length === 2) {
    const [latitude, longitude] = value;
    if (typeof latitude === 'number' && typeof longitude === 'number') {
      return true;
    }
  } else if (typeof value === 'object' && value !== null) {
    if ('latitude' in value && 'longitude' in value) {
      const { latitude, longitude } = value;
      if (typeof latitude === 'number' && typeof longitude === 'number') {
        return true;
      }
    }
  }
  return false;
};

// TODO: implement temporal classifier
function isTemporal(value: any): boolean {
  return false;
  // const datetime = moment(value);
  // if (datetime.isValid()) {
  //   return true;
  // }
  // return false;
}

export const getDimension = (value: any): DimensionType => {
  if (Array.isArray(value)) {
    if (value.length === 0) return 'categorical';
    return getDimension(value[0]);
  }

  if (isTemporal(value)) {
    return 'temporal';
  } else if (isSpatial(value)) {
    return 'spatial';
  } else if (isNumerical(value)) {
    return 'numerical';
  }
  return 'categorical';
};
