import React from 'react';
import { CompositeLayer } from 'deck.gl';
import { ScatterplotLayer } from '@deck.gl/layers';
import { LayerProps } from '../../../mapvis.types';
import { Node } from '@graphpolaris/shared/lib/data-access';

export class NodeLayer extends CompositeLayer<LayerProps> {
  static type = 'Node';

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  getRadius(node: Node, config: any) {
    if (config[node.label]?.fixed) return config[node.label].radius;
    const sizeAttribute = config[node.label]?.sizeAttribute;
    if (sizeAttribute) {
      const minValue = config[node.label]?.min ?? 0;
      const maxValue = config[node.label]?.max ?? 10;
      const attributeValue = parseFloat(node.attributes[sizeAttribute] as string);

      if (!isNaN(attributeValue)) {
        const normalizedValue = (attributeValue - minValue) / (maxValue - minValue);
        return Math.max(1, normalizedValue * 10);
      }
    }
    return config[node.label]?.radius ?? 5;
  }

  renderLayers() {
    const { graph, config, getNodeLocation } = this.props;

    return graph.metaData.nodes.labels.map(
      (label: string) =>
        new ScatterplotLayer({
          id: `${label}-nodes-scatterplot`,
          visible: !config[label].hidden,
          data: graph.nodes.filter((node: Node) => node.label === label),
          pickable: true,
          filled: true,
          radiusScale: 6,
          radiusMinPixels: 7,
          radiusMaxPixels: 100,
          lineWidthMinPixels: 1,
          getPosition: (d: any) => getNodeLocation(d._id),
          getFillColor: (d: any) => config[label].color,
          getRadius: (d: any) => this.getRadius(d, config),
        }),
    );
  }
}
